@@ -5,6 +5,7 @@ The IPython engine application
 
 # Copyright (c) IPython Development Team.
 # Distributed under the terms of the Modified BSD License.
+import asyncio
 import json
 import os
 import signal
@@ -14,6 +15,7 @@ from getpass import getpass
 from io import FileIO, TextIOWrapper
 from logging import StreamHandler
 
+import ipykernel
 import zmq
 from ipykernel.kernelapp import IPKernelApp
 from ipykernel.zmqshell import ZMQInteractiveShell
@@ -52,6 +54,10 @@ from .kernel import IPythonParallelKernel as Kernel
 from .log import EnginePUBHandler
 from .nanny import start_nanny
 
+try:
+    from ipykernel.control import ControlThread
+except ImportError:
+    ControlThread = None
 # -----------------------------------------------------------------------------
 # Module level variables
 # -----------------------------------------------------------------------------
@@ -162,7 +168,7 @@ class IPEngine(BaseParallelApplication):
             base = 'ipcontroller-{}'.format(change['new'])
         else:
             base = 'ipcontroller'
-        self.url_file_name = "%s-engine.json" % base
+        self.url_file_name = f"{base}-engine.json"
 
     log_url = Unicode(
         '',
@@ -270,7 +276,7 @@ class IPEngine(BaseParallelApplication):
             self.log.debug("MPI rank = %i", MPI.COMM_WORLD.rank)
             return MPI.COMM_WORLD.rank
 
-    registrar = Instance('zmq.eventloop.zmqstream.ZMQStream', allow_none=True)
+    registrar = Instance('zmq.Socket', allow_none=True)
     kernel = Instance(Kernel, allow_none=True)
     hb_check_period = Integer()
 
@@ -349,6 +355,8 @@ class IPEngine(BaseParallelApplication):
             self.log.info("Generating new CURVE credentials")
             self.curve_publickey, self.curve_secretkey = zmq.curve_keypair()
 
+    _kernel_start_future = None
+
     def find_connection_file(self):
         """Set the url file.
 
@@ -385,7 +393,9 @@ class IPEngine(BaseParallelApplication):
             self.sshserver = d.get("ssh")
 
         proto, ip = d['interface'].split('://')
+        self.log.debug(f"calling disambiguate_ip_address({ip}, {self.location})")
         ip = disambiguate_ip_address(ip, self.location)
+        self.log.debug(f"disambiguate_ip_address returned ip={ip}")
         d['interface'] = f'{proto}://{ip}'
 
         if d.get('curve_serverkey'):
@@ -414,39 +424,32 @@ class IPEngine(BaseParallelApplication):
         self.log.debug("%r", config)
         self.connection_info = d
 
+    _kernel_bound = False
+
     def bind_kernel(self, **kwargs):
         """Promote engine to listening kernel, accessible to frontends."""
-        if self.kernel_app is not None:
+        if self._kernel_bound:
+            # already run
             return
+        self._kernel_bound = True
 
         self.log.info("Opening ports for direct connections as an IPython kernel")
         if self.curve_serverkey:
             self.log.warning("Bound kernel does not support CURVE security")
 
         kernel = self.kernel
-
-        kwargs.setdefault('config', self.config)
-        kwargs.setdefault('log', self.log)
-        kwargs.setdefault('profile_dir', self.profile_dir)
-        kwargs.setdefault('session', self.session)
-
-        app = self.kernel_app = IPKernelApp(**kwargs)
-
-        # allow IPKernelApp.instance():
-        IPKernelApp._instance = app
+        app = self.kernel_app
 
         app.init_connection_file()
         # relevant contents of init_sockets:
 
-        app.shell_port = app._bind_socket(kernel.shell_streams[0], app.shell_port)
+        app.shell_port = app._bind_socket(self.shell_socket, app.shell_port)
         app.log.debug("shell ROUTER Channel on port: %i", app.shell_port)
 
-        iopub_socket = kernel.iopub_socket
-        # ipykernel 4.3 iopub_socket is an IOThread wrapper:
-        if hasattr(iopub_socket, 'socket'):
-            iopub_socket = iopub_socket.socket
+        app.control_port = app._bind_socket(self.control_socket, app.control_port)
+        app.log.debug("control ROUTER Channel on port: %i", app.control_port)
 
-        app.iopub_port = app._bind_socket(iopub_socket, app.iopub_port)
+        app.iopub_port = app._bind_socket(self.iopub_socket, app.iopub_port)
         app.log.debug("iopub PUB Channel on port: %i", app.iopub_port)
 
         kernel.stdin_socket = self.context.socket(zmq.ROUTER)
@@ -454,7 +457,6 @@ class IPEngine(BaseParallelApplication):
         app.log.debug("stdin ROUTER Channel on port: %i", app.stdin_port)
 
         # start the heartbeat, and log connection info:
-
         app.init_heartbeat()
 
         app.log_connection_info()
@@ -481,7 +483,7 @@ class IPEngine(BaseParallelApplication):
             ):
                 password = False
             else:
-                password = getpass("SSH Password for %s: " % self.sshserver)
+                password = getpass(f"SSH Password for {self.sshserver}: ")
         else:
             password = False
 
@@ -523,7 +525,7 @@ class IPEngine(BaseParallelApplication):
 
         return connect, maybe_tunnel
 
-    def register(self):
+    async def register(self):
         """send the registration_request"""
         if self.use_mpi and self.id and self.id >= 100 and self.mpi_registration_delay:
             # Some launchres implement delay at the Launcher level,
@@ -536,39 +538,70 @@ class IPEngine(BaseParallelApplication):
             )
             time.sleep(delay)
 
-        self.log.info("Registering with controller at %s" % self.registration_url)
+        self.log.info(f"Registering with controller at {self.registration_url}")
         ctx = self.context
         connect, maybe_tunnel = self.init_connector()
-        reg = ctx.socket(zmq.DEALER)
-        reg.setsockopt(zmq.IDENTITY, self.bident)
+        reg = self.registrar = ctx.socket(zmq.DEALER)
+        reg.IDENTITY = self.bident
         connect(reg, self.registration_url)
 
-        self.registrar = zmqstream.ZMQStream(reg, self.loop)
-
         content = dict(uuid=self.ident)
         if self.id is not None:
             self.log.info("Requesting id: %i", self.id)
             content['id'] = self.id
-        self._registration_completed = False
-        self.registrar.on_recv(
-            lambda msg: self.complete_registration(msg, connect, maybe_tunnel)
-        )
 
-        self.session.send(self.registrar, "registration_request", content=content)
+        self.session.send(reg, "registration_request", content=content)
+        # wait for reply
+        poller = zmq.asyncio.Poller()
+        poller.register(reg, zmq.POLLIN)
+        events = dict(await poller.poll(timeout=int(self.timeout * 1_000)))
+        if events:
+            msg = reg.recv_multipart()
+            try:
+                await self.complete_registration(msg, connect, maybe_tunnel)
+            except Exception as e:
+                self.log.critical(f"Error completing registration: {e}", exc_info=True)
+                self.exit(255)
+        else:
+            self.abort()
 
     def _report_ping(self, msg):
         """Callback for when the heartmonitor.Heart receives a ping"""
         # self.log.debug("Received a ping: %s", msg)
         self._hb_last_pinged = time.time()
 
-    def complete_registration(self, msg, connect, maybe_tunnel):
-        try:
-            self._complete_registration(msg, connect, maybe_tunnel)
-        except Exception as e:
-            self.log.critical(f"Error completing registration: {e}", exc_info=True)
-            self.exit(255)
+    def redirect_output(self, iopub_socket):
+        """Redirect std streams and set a display hook."""
+        if self.out_stream_factory:
+            sys.stdout = self.out_stream_factory(self.session, iopub_socket, 'stdout')
+            sys.stdout.topic = f"engine.{self.id}.stdout".encode("ascii")
+            sys.stderr = self.out_stream_factory(self.session, iopub_socket, 'stderr')
+            sys.stderr.topic = f"engine.{self.id}.stderr".encode("ascii")
 
-    def _complete_registration(self, msg, connect, maybe_tunnel):
+            # copied from ipykernel 6, which captures sys.__stderr__ at the FD-level
+            if getattr(sys.stderr, "_original_stdstream_copy", None) is not None:
+                for handler in self.log.handlers:
+                    print(handler)
+                    if isinstance(handler, StreamHandler) and (
+                        handler.stream.buffer.fileno()
+                    ):
+                        self.log.debug(
+                            "Seeing logger to stderr, rerouting to raw filedescriptor."
+                        )
+
+                        handler.stream = TextIOWrapper(
+                            FileIO(sys.stderr._original_stdstream_copy, "w")
+                        )
+        if self.display_hook_factory:
+            sys.displayhook = self.display_hook_factory(self.session, iopub_socket)
+            sys.displayhook.topic = f"engine.{self.id}.execute_result".encode("ascii")
+
+    def restore_output(self):
+        """Restore output after redirect_output"""
+        sys.stdout = sys.__stdout__
+        sys.stderr = sys.__stderr__
+
+    async def complete_registration(self, msg, connect, maybe_tunnel):
         ctx = self.context
         loop = self.loop
         identity = self.bident
@@ -579,7 +612,7 @@ class IPEngine(BaseParallelApplication):
 
         def url(key):
             """get zmq url for given channel"""
-            return str(info["interface"] + ":%i" % info[key])
+            return f"{info['interface']}:{info[key]}"
 
         def urls(key):
             return [f'{info["interface"]}:{port}' for port in info[key]]
@@ -592,7 +625,7 @@ class IPEngine(BaseParallelApplication):
                     f"Did not get the requested id: {self.id} != {requested_id}"
                 )
                 self.log.name = self.log.name.rsplit(".", 1)[0] + f".{self.id}"
-            elif self.id is None:
+            elif self.id is not None:
                 self.log.name += f".{self.id}"
 
             # create Shell Connections (MUX, Task, etc.):
@@ -608,16 +641,15 @@ class IPEngine(BaseParallelApplication):
             self.log.info(f'Shell_addrs: {shell_addrs}')
 
             # Use only one shell stream for mux and tasks
-            stream = zmqstream.ZMQStream(ctx.socket(zmq.ROUTER), loop)
-            stream.setsockopt(zmq.IDENTITY, identity)
+            shell_socket = self.shell_socket = ctx.socket(zmq.ROUTER)
+            shell_socket.setsockopt(zmq.IDENTITY, identity)
             # TODO: enable PROBE_ROUTER when schedulers can handle the empty message
             # stream.setsockopt(zmq.PROBE_ROUTER, 1)
             self.log.debug("Setting shell identity %r", identity)
 
-            shell_streams = [stream]
             for addr in shell_addrs:
                 self.log.info("Connecting shell to %s", addr)
-                connect(stream, addr)
+                connect(shell_socket, addr)
 
             # control stream:
             control_url = url('control')
@@ -629,13 +661,13 @@ class IPEngine(BaseParallelApplication):
                 control_url = nanny_url
                 # nanny uses our curve_publickey, not the controller's publickey
                 curve_serverkey = self.curve_publickey
-            control_stream = zmqstream.ZMQStream(ctx.socket(zmq.ROUTER), loop)
-            control_stream.setsockopt(zmq.IDENTITY, identity)
-            connect(control_stream, control_url, curve_serverkey=curve_serverkey)
+            control_socket = self.control_socket = ctx.socket(zmq.ROUTER)
+            control_socket.setsockopt(zmq.IDENTITY, identity)
+            connect(control_socket, control_url, curve_serverkey=curve_serverkey)
 
             # create iopub stream:
             iopub_addr = url('iopub')
-            iopub_socket = ctx.socket(zmq.PUB)
+            iopub_socket = self.iopub_socket = ctx.socket(zmq.PUB)
             iopub_socket.SNDHWM = 0
             iopub_socket.setsockopt(zmq.IDENTITY, identity)
             connect(iopub_socket, iopub_addr)
@@ -650,36 +682,6 @@ class IPEngine(BaseParallelApplication):
             # disable history:
             self.config.HistoryManager.hist_file = ':memory:'
 
-            # Redirect input streams and set a display hook.
-            if self.out_stream_factory:
-                sys.stdout = self.out_stream_factory(
-                    self.session, iopub_socket, 'stdout'
-                )
-                sys.stdout.topic = f"engine.{self.id}.stdout".encode("ascii")
-                sys.stderr = self.out_stream_factory(
-                    self.session, iopub_socket, 'stderr'
-                )
-                sys.stderr.topic = f"engine.{self.id}.stderr".encode("ascii")
-
-                # copied from ipykernel 6, which captures sys.__stderr__ at the FD-level
-                if getattr(sys.stderr, "_original_stdstream_copy", None) is not None:
-                    for handler in self.log.handlers:
-                        if isinstance(handler, StreamHandler) and (
-                            handler.stream.buffer.fileno() == 2
-                        ):
-                            self.log.debug(
-                                "Seeing logger to stderr, rerouting to raw filedescriptor."
-                            )
-
-                            handler.stream = TextIOWrapper(
-                                FileIO(sys.stderr._original_stdstream_copy, "w")
-                            )
-            if self.display_hook_factory:
-                sys.displayhook = self.display_hook_factory(self.session, iopub_socket)
-                sys.displayhook.topic = f"engine.{self.id}.execute_result".encode(
-                    "ascii"
-                )
-
             # patch Session to always send engine uuid metadata
             original_send = self.session.send
 
@@ -713,17 +715,28 @@ class IPEngine(BaseParallelApplication):
 
             self.session.send = send_with_metadata
 
+            kernel_kwargs = {}
+            if ipykernel.version_info >= (6,):
+                kernel_kwargs["control_thread"] = control_thread = ControlThread(
+                    daemon=True
+                )
+
+            control_thread.start()
+            kernel_kwargs["control_stream"] = zmqstream.ZMQStream(
+                control_socket, control_thread.io_loop
+            )
+
+            kernel_kwargs["shell_streams"] = [zmqstream.ZMQStream(shell_socket)]
+
             self.kernel = Kernel.instance(
                 parent=self,
                 engine_id=self.id,
                 ident=self.ident,
                 session=self.session,
-                control_stream=control_stream,
-                shell_streams=shell_streams,
                 iopub_socket=iopub_socket,
-                loop=loop,
                 user_ns=self.user_ns,
                 log=self.log,
+                **kernel_kwargs,
             )
 
             self.kernel.shell.display_pub.topic = f"engine.{self.id}.displaypub".encode(
@@ -731,19 +744,30 @@ class IPEngine(BaseParallelApplication):
             )
 
             # FIXME: This is a hack until IPKernelApp and IPEngineApp can be fully merged
-            self.init_signal()
-            app = IPKernelApp(
+            app = self.kernel_app = IPKernelApp(
                 parent=self, shell=self.kernel.shell, kernel=self.kernel, log=self.log
             )
+            # allow IPKernelApp.instance():
+            IPKernelApp._instance = app
+            self.init_signal()
+
             if self.use_mpi and self.init_mpi:
                 app.exec_lines.insert(0, self.init_mpi)
             app.init_profile_dir()
+            app.init_gui_pylab()
+            app.init_extensions()
             app.init_code()
 
-            self.kernel.start()
+            # redirect output at the end, only after start is called
+            self.redirect_output(iopub_socket)
+
+            # ipykernel 7, kernel.start is the long-running main loop
+            start_future = self.kernel.start()
+            if start_future is not None:
+                self._kernel_start_future = asyncio.ensure_future(start_future)
         else:
-            self.log.fatal("Registration Failed: %s" % msg)
-            raise Exception("Registration Failed: %s" % msg)
+            self.log.fatal(f"Registration Failed: {msg}")
+            raise Exception(f"Registration Failed: {msg}")
 
         self.start_heartbeat(
             maybe_tunnel(url('hb_ping')),
@@ -751,8 +775,7 @@ class IPEngine(BaseParallelApplication):
             content['hb_period'],
             identity,
         )
-        self.log.info("Completed registration with id %i" % self.id)
-        self.loop.remove_timeout(self._abort_timeout)
+        self.log.info("Completed registration with id %i", self.id)
 
     def start_nanny(self, control_url):
         self.log.info("Starting nanny")
@@ -779,12 +802,12 @@ class IPEngine(BaseParallelApplication):
             if self.curve_serverkey:
                 mon.setsockopt(zmq.CURVE_SERVER, 1)
                 mon.setsockopt(zmq.CURVE_SECRETKEY, self.curve_secretkey)
-            mport = mon.bind_to_random_port('tcp://%s' % localhost())
+            mport = mon.bind_to_random_port(f'tcp://{localhost()}')
             mon.setsockopt(zmq.SUBSCRIBE, b"")
             self._hb_listener = zmqstream.ZMQStream(mon, self.loop)
             self._hb_listener.on_recv(self._report_ping)
 
-            hb_monitor = "tcp://%s:%i" % (localhost(), mport)
+            hb_monitor = f"tcp://{localhost()}:{mport}"
 
         heart = Heart(
             hb_ping,
@@ -817,7 +840,7 @@ class IPEngine(BaseParallelApplication):
             )
 
     def abort(self):
-        self.log.fatal("Registration timed out after %.1f seconds" % self.timeout)
+        self.log.fatal(f"Registration timed out after {self.timeout:.1f} seconds")
         if "127." in self.registration_url:
             self.log.fatal(
                 """
@@ -889,13 +912,13 @@ class IPEngine(BaseParallelApplication):
 
         exec_lines = []
         for app in ('IPKernelApp', 'InteractiveShellApp'):
-            if '%s.exec_lines' % app in config:
+            if f'{app}.exec_lines' in config:
                 exec_lines = config[app].exec_lines
                 break
 
         exec_files = []
         for app in ('IPKernelApp', 'InteractiveShellApp'):
-            if '%s.exec_files' % app in config:
+            if f'{app}.exec_files' in config:
                 exec_files = config[app].exec_files
                 break
 
@@ -919,6 +942,9 @@ class IPEngine(BaseParallelApplication):
 
     @catch_config_error
     def initialize(self, argv=None):
+        if "PYDEVD_DISABLE_FILE_VALIDATION" not in os.environ:
+            # suppress irrelevant debugger warnings by default
+            os.environ["PYDEVD_DISABLE_FILE_VALIDATION"] = "1"
         super().initialize(argv)
         self.init_engine()
         self.forward_logging()
@@ -932,24 +958,60 @@ class IPEngine(BaseParallelApplication):
 
     def _signal_stop(self, sig, frame):
         self.log.critical(f"received signal {sig}, stopping")
-        self.loop.add_callback_from_signal(self.loop.stop)
+        # we are shutting down, stop forwarding output
+        try:
+            self.restore_output()
+            # kernel.stop added in ipykernel 7
+            # claims to be threadsafe, but is not
+            kernel_stop = getattr(self.kernel, "stop", None)
+            if kernel_stop is not None:
+                self.log.debug("Calling kernel.stop()")
+
+                # callback must be async for event loop to be
+                # detected by anyio
+                async def stop():
+                    # guard against kernel stop being made async
+                    # in the future. It is sync in 7.0
+                    f = kernel_stop()
+                    if f is not None:
+                        await f
+
+                self.loop.add_callback_from_signal(stop)
+            if self._kernel_start_future is None:
+                # not awaiting start_future, stop loop directly
+                self.log.debug("Stopping event loop")
+                self.loop.add_callback_from_signal(self.loop.stop)
+        except Exception:
+            self.log.critical("Failed to stop kernel", exc_info=True)
+            self.loop.add_callback_from_signal(self.loop.stop)
 
     def start(self):
         if self.id is not None:
             self.log.name += f".{self.id}"
-        loop = self.loop
-
-        def _start():
-            self.register()
-            self._abort_timeout = loop.add_timeout(
-                loop.time() + self.timeout, self.abort
-            )
-
-        self.loop.add_callback(_start)
         try:
-            self.loop.start()
-        except KeyboardInterrupt:
-            self.log.critical("Engine Interrupted, shutting down...\n")
+            self.loop.run_sync(self._start)
+        except (asyncio.TimeoutError, KeyboardInterrupt):
+            # tornado run_sync raises TimeoutError
+            # if the task didn't finish
+            pass
+
+    async def _start(self):
+        await self.register()
+        # run forever
+        if self._kernel_start_future is None:
+            while True:
+                try:
+                    await asyncio.sleep(60)
+                except asyncio.CancelledError:
+                    pass
+        else:
+            self.log.info("awaiting start future")
+            try:
+                await self._kernel_start_future
+            except asyncio.CancelledError:
+                pass
+            except Exception as e:
+                self.log.critical("Error awaiting start future", exc_info=True)
 
 
 main = launch_new_instance = IPEngine.launch_instance
