@@ -11,11 +11,9 @@ from unittest.mock import MagicMock, create_autospec, patch
 
 import pytest
 import zmq
-from ipykernel import iostream, kernelapp
+from ipykernel import kernelapp
 from ipykernel.ipkernel import IPythonKernel
 from IPython.core.profiledir import ProfileDir
-from tornado import ioloop
-from zmq.eventloop import zmqstream
 
 import ipyparallel as ipp
 import ipyparallel.engine.app
@@ -37,7 +35,7 @@ def _get_output(cmd):
     ],
 )
 def test_version(submod):
-    out = _get_output([sys.executable, '-m', 'ipyparallel.%s' % submod, '--version'])
+    out = _get_output([sys.executable, '-m', f'ipyparallel.{submod}', '--version'])
     assert out.strip() == ipyparallel.__version__
 
 
@@ -50,7 +48,7 @@ def test_version(submod):
     ],
 )
 def test_help_all(submod):
-    out = _get_output([sys.executable, '-m', 'ipyparallel.%s' % submod, '--help-all'])
+    out = _get_output([sys.executable, '-m', f'ipyparallel.{submod}', '--help-all'])
 
 
 @pytest.mark.parametrize(
@@ -70,22 +68,18 @@ def test_ipcluster_help_all(subcommand):
 
 
 def bind_kernel(engineapp):
-    app = MagicMock(spec=kernelapp.IPKernelApp)
-    with patch.object(
-        ipyparallel.engine.app, 'IPKernelApp', autospec=True
-    ) as MockKernelApp:
-        MockKernelApp.return_value = app
-        app.shell_port = app.iopub_port = app.stdin_port = 0
-        app._bind_socket = types.MethodType(kernelapp.IPKernelApp._bind_socket, app)
-        if hasattr(kernelapp.IPKernelApp, '_try_bind_socket'):
-            app._try_bind_socket = types.MethodType(
-                kernelapp.IPKernelApp._try_bind_socket,
-                app,
-            )
-        app.transport = 'tcp'
-        app.ip = '127.0.0.1'
-        app.init_heartbeat.return_value = None
-        engineapp.bind_kernel()
+    app = engineapp.kernel_app
+    app.shell_port = app.iopub_port = app.control_port = app.stdin_port = 0
+    app._bind_socket = types.MethodType(kernelapp.IPKernelApp._bind_socket, app)
+    if hasattr(kernelapp.IPKernelApp, '_try_bind_socket'):
+        app._try_bind_socket = types.MethodType(
+            kernelapp.IPKernelApp._try_bind_socket,
+            app,
+        )
+    app.transport = 'tcp'
+    app.ip = '127.0.0.1'
+    app.init_heartbeat.return_value = None
+    engineapp.bind_kernel()
 
 
 def test_bind_kernel(request):
@@ -97,7 +91,9 @@ def test_bind_kernel(request):
         context = ctx
 
     app = MockIPEngineApp()
-    app.kernel_app = None
+    app.kernel_app = MagicMock(spec=kernelapp.IPKernelApp)
+    # kernelapp.IPKernelApp._instance = app.kernel_app
+
     app.kernel = MagicMock(spec=IPythonKernel)
 
     def socket_spec():
@@ -105,44 +101,14 @@ def test_bind_kernel(request):
         spec.FD = 2
         return spec
 
-    app.kernel.shell_streams = [
-        zmqstream.ZMQStream(
-            socket=socket_spec(),
-            io_loop=create_autospec(spec=ioloop.IOLoop, spec_set=True, instance=True),
-        )
-    ]
+    app.shell_socket = socket_spec()
+    app.control_socket = socket_spec()
+    app.iopub_socket = socket_spec()
 
-    app.kernel.control_stream = zmqstream.ZMQStream(
-        socket=socket_spec(),
-        io_loop=create_autospec(spec=ioloop.IOLoop, spec_set=True, instance=True),
-    )
-
-    # testing the case iopub_socket is not replaced with IOPubThread
-    iopub_socket = socket_spec()
-
-    app.kernel.iopub_socket = iopub_socket
-    assert isinstance(app.kernel.iopub_socket, zmq.Socket)
     bind_kernel(app)
     assert (
-        app.kernel.iopub_socket.bind_to_random_port.called
-        and app.kernel.iopub_socket.bind_to_random_port.call_count == 1
-    )
-
-    # testing the case iopub_socket is replaced with IOPubThread
-    class TestIOPubThread(iostream.IOPubThread):
-        socket = None
-
-    iopub_socket.reset_mock()
-    app.kernel_app = None
-    app.kernel.iopub_socket = create_autospec(
-        spec=TestIOPubThread, spec_set=True, instance=True
-    )
-    app.kernel.iopub_socket.socket = iopub_socket
-    assert isinstance(app.kernel.iopub_socket, iostream.IOPubThread)
-    bind_kernel(app)
-    assert (
-        app.kernel.iopub_socket.socket.bind_to_random_port.called
-        and app.kernel.iopub_socket.socket.bind_to_random_port.call_count == 1
+        app.iopub_socket.bind_to_random_port.called
+        and app.iopub_socket.bind_to_random_port.call_count == 1
     )
 
 
