@@ -613,13 +613,6 @@ class TestClient(ClusterTestCase):
         assert f.result() == 'future'
 
     @skip_without('distributed')
-    @pytest.mark.skipif(
-        sys.version_info[:2] <= (3, 5), reason="become_dask doesn't work on Python 3.5"
-    )
-    @pytest.mark.skipif(
-        tornado.version_info[:2] < (5,),
-        reason="become_dask doesn't work with tornado 4",
-    )
     @pytest.mark.filterwarnings("ignore:make_current")
     def test_become_dask(self):
         executor = self.client.become_dask()
