@@ -173,18 +173,18 @@ class TestParallelMagics(ClusterTestCase):
         expected.extend(
             [
                 r'\[output:\d+\]',
-                'IPython.core.display.HTML',
+                'IPython..+.HTML',
             ]
             * len(v)
         )
         expected.extend(
             [
                 r'\[output:\d+\]',
-                'IPython.core.display.Math',
+                'IPython..+.Math',
             ]
             * len(v)
         )
-        expected.extend([r'Out\[\d+:\d+\]:.*IPython\.core\.display\.Math'] * len(v))
+        expected.extend([r'Out\[\d+:\d+\]:.*IPython\..+\.Math'] * len(v))
 
         assert len(lines), len(expected) == io.stdout
         for line, expect in zip(lines, expected):
@@ -286,6 +286,7 @@ class TestParallelMagics(ClusterTestCase):
         printed_tb = "\n".join(exc_info.value.render_traceback())
         assert printed_tb.count("RuntimeError:") >= ipp.error.CompositeError.tb_limit
 
+    @pytest.mark.skip("ordering issues in ipykernel 7")
     def test_cellpx_stream(self):
         """%%px --stream"""
         self.minimum_engines(6)
@@ -323,9 +324,9 @@ class TestParallelMagics(ClusterTestCase):
         # Check that all expected lines are in the output
         self._check_expected_lines_unordered(expected, lines)
 
-        assert (
-            len(expected) - len(v) <= len(lines) <= len(expected)
-        ), f"expected {len(expected)} lines, got: {io.stdout}"
+        assert len(expected) - len(v) <= len(lines) <= len(expected), (
+            f"expected {len(expected)} lines, got: {io.stdout}"
+        )
 
         # Do the same for stderr
         print(io.stderr, file=sys.stderr)
@@ -342,9 +343,9 @@ class TestParallelMagics(ClusterTestCase):
             * len(v)
         )
         self._check_expected_lines_unordered(expected, lines)
-        assert (
-            len(expected) - len(v) <= len(lines) <= len(expected)
-        ), f"expected {len(expected)} lines, got: {io.stderr}"
+        assert len(expected) - len(v) <= len(lines) <= len(expected), (
+            f"expected {len(expected)} lines, got: {io.stderr}"
+        )
 
     def test_px_nonblocking(self):
         ip = get_ipython()
@@ -439,8 +440,8 @@ class TestParallelMagics(ClusterTestCase):
                 ip.run_line_magic('pxresult', '')
             assert str(data[name]) in io.stdout
 
-    def test_px_pylab(self):
-        """%pylab works on engines"""
+    def test_px_matplotlib(self):
+        """%matplotlib inline works on engines"""
         pytest.importorskip('matplotlib')
         ip = get_ipython()
         v = self.client[-1]
@@ -448,15 +449,19 @@ class TestParallelMagics(ClusterTestCase):
         v.activate()
 
         with capture_output() as io:
-            ip.run_line_magic("px", "%pylab inline")
-
-        assert (
-            "Populating the interactive namespace from numpy and matplotlib"
-            in io.stdout
-        )
+            ip.run_line_magic(
+                "px",
+                "\n".join(
+                    [
+                        "%matplotlib inline",
+                        "import numpy as np",
+                        "import matplotlib.pyplot as plt",
+                    ]
+                ),
+            )
 
         with capture_output(display=False) as io:
-            ip.run_line_magic("px", "plot(rand(100))")
+            ip.run_line_magic("px", "plt.plot(np.random.rand(100))")
         assert 'Out[' in io.stdout
         assert 'matplotlib.lines' in io.stdout
 
