@@ -329,7 +329,7 @@ class TestView(ClusterTestCase):
         view.push(dict(ts=ts), block=True)
         rts = view['ts']
 
-        assert type(rts) == type(ts)
+        assert type(rts) is type(ts)
         assert (ts == rts).all()
 
     def test_map(self):
@@ -453,10 +453,7 @@ class TestView(ClusterTestCase):
         """test executing unicode strings"""
         v = self.client[-1]
         v.block = True
-        if sys.version_info[0] >= 3:
-            code = "a='é'"
-        else:
-            code = "a=u'é'"
+        code = "a='é'"
         v.execute(code)
         assert v['a'] == 'é'
 
@@ -472,8 +469,8 @@ class TestView(ClusterTestCase):
 
         @interactive
         def check_unicode(a, check):
-            assert not isinstance(a, bytes), "%r is bytes, not unicode" % a
-            assert isinstance(check, bytes), "%r is not bytes" % check
+            assert not isinstance(a, bytes), f"{a!r} is bytes, not unicode"
+            assert isinstance(check, bytes), f"{check!r} is not bytes"
             assert a.encode('utf8') == check, f"{a} != {check}"
 
         for s in ['é', 'ßø®∫', 'asdf']:
@@ -556,7 +553,7 @@ class TestView(ClusterTestCase):
         e0.block = True
         ar = e0.execute("5", silent=False)
         er = ar.get()
-        assert str(er) == "<ExecuteReply[%i]: 5>" % er.execution_count
+        assert str(er) == f"<ExecuteReply[{er.execution_count}]: 5>"
         assert er.execute_result['data']['text/plain'] == '5'
 
     def test_execute_reply_rich(self):
@@ -612,12 +609,12 @@ class TestView(ClusterTestCase):
                 if split == ['a', 'int', '5']:
                     found = True
                     break
-            assert found, "whos output wrong: %s" % stdout
+            assert found, f"whos output wrong: {stdout}"
 
     def test_execute_displaypub(self):
         """execute tracks display_pub output"""
         view = self.client[:]
-        view.execute("from IPython.core.display import *")
+        view.execute("from IPython.display import *")
         ar = view.execute("[ display(i) for i in range(5) ]", block=True)
 
         expected = [{'text/plain': str(j)} for j in range(5)]
@@ -628,7 +625,7 @@ class TestView(ClusterTestCase):
     def test_apply_displaypub(self):
         """apply tracks display_pub output"""
         view = self.client[:]
-        view.execute("from IPython.core.display import *")
+        view.execute("from IPython.display import *")
 
         @interactive
         def publish():
@@ -936,9 +933,9 @@ class TestView(ClusterTestCase):
         view = self.client[:]
         # enable cloudpickle
         view.use_cloudpickle()
-        assert (
-            'types' not in globals()
-        ), "Test condition isn't met if types is already imported"
+        assert 'types' not in globals(), (
+            "Test condition isn't met if types is already imported"
+        )
 
         @ipp.require("types")
         @ipp.interactive
